function [soma, fg, fg_threshold] = detect_soma( img, soma_size, max_neurite_width )
% some_size is the minimal size of a soma
% max_neurite_width is the max width of a neurite protruded from a soma
% Both soma_size and max_neurite_width depend on the imaging protocol and
% the nature of neuronal cells. They should be decided during the pilot
% study for large-scale high-content neuronal screens.

if nargin < 2
    soma_size = 100;
end
if nargin < 3
    max_neurite_width = 4;
end

% detect foreground (fg), assume gaussian distribution of background
[c, x] = imhist(img);
bk_mean = x( c == max(c) );
v = sqrt( sum( (img(:) - bk_mean).^2) / (length(img(:))-1) );
fg = img > bk_mean + 3*v; % sometimes, 2*v may be better depending on the noisy level
preV = -10;
while abs(v - preV) > 0.01
    preV = v;
    temp = img( fg == 0 );
    temp = temp(:);
    v = sqrt( sum( (temp-bk_mean) .^ 2) / (length(temp)-1) );
    fg = img > bk_mean + 3*v;
end
fg_threshold = bk_mean + 3*v;

% The following step is necessary because the above background distribution 
% estimation process can be off if there is little or no foreground at all.
% The threshold lower bound is set as 0.115 of [0, 1] ~= 30 of [0, 255]. 
% This lower bound is affected by the imaging protocol and the imaging instrument.
% It should be decided emperically.
if fg_threshold < 0.115 
    fg_threshold = 0.115;
end
fg = img > fg_threshold;

% fill holes -- soma are darker in peripheral -> create internal holes 
temp = 1 - fg;
temp = temp - bwareaopen( temp, round(soma_size/2.5), 4 ); % 2.5 is an emperical value 
fg = fg + temp;

% remove neurites. use max_neurite_width for individual small cells
soma = imopen( fg, strel('disk', max_neurite_width) );

% remove small islands due to noise
soma = bwareaopen( soma, soma_size );

% for those > 2 * soma, reduce the effects of neurite bundles using max_neurite_width+2
large_soma = bwareaopen( soma, soma_size*2 );
small_soma = soma - large_soma;
large_soma = imopen( large_soma, strel('disk', max_neurite_width+2) );

soma = small_soma + large_soma;